const fetch = require("node-fetch");
const { ApolloServer, gql } = require("apollo-server");

// Esta es una colección (de muestra) de libros que podremos consultar
// el servidor GraphQL para. Un ejemplo más completo podría recuperar
// de una fuente de datos existente como una API REST o una base de datos.
const books = [
  {
    title: "Harry Potter and the Chamber of Secrets",
    author: "J.K. Rowling"
  },
  {
    title: "Jurassic Park",
    author: "Michael Crichton"
  },
  {
    title:"El perfume",
    author:"Patrick"
  }
];

// Las definiciones de tipo definen la "forma" de sus datos y especifican
// de qué formas se pueden obtener los datos del servidor GraphQL.
const typeDefs = gql`
  # Comments in GraphQL are defined with the hash (#) symbol.

  # This "Book" type can be used in other type declarations.
  type Book {
    title: String
    author: String
  }
  enum CharacterStatus{
    Alive
    Dead
    unknown
  }
  type Character{
    name: String
    id: ID
    status: CharacterStatus
    episodes: [String]
  }

  # The "Query" type is the root of all GraphQL queries.
  # (A "Mutation" type will be covered later on.)
  type Query {
    books: [Book]
    characters: [Character]
    character(id: ID): Character
  }
`;

// Los resolutores definen la técnica para obtener los tipos en el
// esquema. Recuperamos libros de la matriz "libros" de arriba.
const resolvers = {
  Query: {
    books: () => books,
    characters: () => fetchCharacters(),
    character: (parent, args) =>{
      const {id} = args
      return fetchCharacter({id})
    }
  }
};

// En el sentido más básico, ApolloServer se puede iniciar
// pasando definiciones de tipo (typeDefs) y los resolutores
// responsable de obtener los datos de esos tipos.
const server = new ApolloServer({ typeDefs, resolvers });

// Este método `listen` lanza un servidor web. Aplicaciones existentes
// puede utilizar opciones de middleware, que discutiremos más adelante.
server.listen().then(({ url }) => {
  console.log(`🚀  Server ready at ${url}`);
});

// datos simulados para personajes
const characters = [
  {
    name: "Rick Sanchez",
    id: 1,
    status: "Alive",
    episodes: [
      "https://rickandmortyapi.com/api/episode/1",
      "https://rickandmortyapi.com/api/episode/2"
    ]
  },
  {
    name: "Morty Smith",
    id: 2,
    status: "Alive",
    episodes: [
      "https://rickandmortyapi.com/api/episode/1",
      "https://rickandmortyapi.com/api/episode/3"
    ]
  }
];

// mock data for episodes
const episodes = [
  {
    name: "Pilot",
    id: 1
  },
  {
    name: "Lawnmower Dog",
    id: 2
  }
];

function fetchEpisodes() {
// ¿Más información sobre la función de recuperación? https://github.com/bitinn/node-fetch#json
  return fetch("https://rickandmortyapi.com/api/episode/")
    .then(res => res.json())
    .then(json => json.results);
}

function fetchEpisodeById(id) {
  return fetch("https://rickandmortyapi.com/api/episode/" + id)
    .then(res => res.json())
    .then(json => json);
}

function fetchEpisodeByUrl(url) {
  return fetch(url)
    .then(res => res.json())
    .then(json => json);
}

async function fetchCharacter() {
  // ¿Más información sobre la función de recuperación? https://github.com/bitinn/node-fetch#json
    const res = await fetch("https://rickandmortyapi.com/api/character/" + id);
  return res.json();
 
  }

function fetchCharacters() {
// ¿Más información sobre la función de recuperación? https://github.com/bitinn/node-fetch#json
  return fetch("https://rickandmortyapi.com/api/character/")
    .then(res => res.json())
    .then(json => json.results);
}

function fetchCharacterById(id) {
  // ¿Más información sobre la función de recuperación? https://github.com/bitinn/node-fetch#json
  return fetch("https://rickandmortyapi.com/api/character/" + id)
    .then(res => res.json())
    .then(json => json);
}

function fetchCharacterByUrl(url) {
  // ¿Más información sobre la función de recuperación? https://github.com/bitinn/node-fetch#json
  return fetch(url)
    .then(res => res.json())
    .then(json => json);
}
